@@ -1,6 +1,6 @@
 # coding: utf-8
 
-# Copyright 2014-2015 Álvaro Justen <https://github.com/turicas/rows/>
+# Copyright 2014-2016 Álvaro Justen <https://github.com/turicas/rows/>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 from __future__ import unicode_literals
 
+import binascii
 import collections
 import datetime
 import json
@@ -24,16 +25,20 @@ import locale
 import re
 import types
 
+from base64 import b64decode, b64encode
 from decimal import Decimal, InvalidOperation
 
+import six
+
 
 # Order matters here
 __all__ = ['BoolField', 'IntegerField', 'FloatField', 'DatetimeField',
            'DateField', 'DecimalField', 'PercentField', 'JSONField',
            'EmailField', 'TextField', 'BinaryField', 'Field']
-REGEXP_ONLY_NUMBERS = re.compile('[^0-9]')
+REGEXP_ONLY_NUMBERS = re.compile('[^0-9\-]')
 SHOULD_NOT_USE_LOCALE = True  # This variable is changed by rows.locale_manager
-NULL = (b'-', b'null', b'none', b'nil', b'n/a', b'na')
+NULL = ('-', 'null', 'none', 'nil', 'n/a', 'na')
+NULL_BYTES = (b'-', b'null', b'none', b'nil', b'n/a', b'na')
 
 
 class Field(object):
@@ -43,7 +48,7 @@ class Field(object):
     actually implements what is expected in the BinaryField
     """
 
-    TYPE = (types.NoneType, )
+    TYPE = (type(None), )
 
     @classmethod
     def serialize(cls, value, *args, **kwargs):
@@ -79,22 +84,35 @@ class BinaryField(Field):
     Is not locale-aware (does not need to be)
     """
 
-    TYPE = (types.StringType, )
+    TYPE = (six.binary_type, )
 
     @classmethod
     def serialize(cls, value, *args, **kwargs):
-        if value is None:
-            value = ''
-
-        return cls.TYPE[0](value)
-
+        if value is not None:
+            if not isinstance(value, six.binary_type):
+                raise ValueError("Can't be {}".format(cls.__name__))
+            else:
+                try:
+                    return b64encode(value).decode('ascii')
+                except (TypeError, binascii.Error):
+                    return value
+        else:
+            return ''
 
     @classmethod
     def deserialize(cls, value, *args, **kwargs):
-        if value is None:
-            return None
+        if value is not None:
+            if isinstance(value, six.binary_type):
+                return value
+            elif isinstance(value, six.text_type):
+                try:
+                    return b64decode(value)
+                except (TypeError, ValueError, binascii.Error):
+                    raise ValueError("Can't decode base64")
+            else:
+                raise ValueError("Can't be {}".format(cls.__name__))
         else:
-            return cls.TYPE[0](value)
+            return b''
 
 
 class BoolField(Field):
@@ -104,7 +122,7 @@ class BoolField(Field):
     attributes like `TRUE_VALUES` and `FALSE_VALUES`)
     """
 
-    TYPE = (types.BooleanType, )
+    TYPE = (bool, )
     SERIALIZED_VALUES = {True: 'true', False: 'false', None: ''}
     TRUE_VALUES = ('true', 'yes')
     FALSE_VALUES = ('false', 'no')
@@ -135,7 +153,7 @@ class IntegerField(Field):
     Is locale-aware
     """
 
-    TYPE = (types.IntType, )
+    TYPE = (int, )
 
     @classmethod
     def serialize(cls, value, *args, **kwargs):
@@ -143,7 +161,7 @@ class IntegerField(Field):
             return ''
 
         if SHOULD_NOT_USE_LOCALE:
-            return types.UnicodeType(value)
+            return six.text_type(value)
         else:
             grouping = kwargs.get('grouping', None)
             return locale.format('%d', value, grouping=grouping)
@@ -171,7 +189,7 @@ class FloatField(Field):
     Is locale-aware
     """
 
-    TYPE = (types.FloatType, )
+    TYPE = (float, )
 
     @classmethod
     def serialize(cls, value, *args, **kwargs):
@@ -179,7 +197,7 @@ class FloatField(Field):
             return ''
 
         if SHOULD_NOT_USE_LOCALE:
-            return types.UnicodeType(value)
+            return six.text_type(value)
         else:
             grouping = kwargs.get('grouping', None)
             return locale.format('%f', value, grouping=grouping)
@@ -210,7 +228,7 @@ class DecimalField(Field):
         if value is None:
             return ''
 
-        value_as_string = types.UnicodeType(value)
+        value_as_string = six.text_type(value)
         if SHOULD_NOT_USE_LOCALE:
             return value_as_string
         else:
@@ -229,7 +247,7 @@ class DecimalField(Field):
         if value is None or isinstance(value, cls.TYPE):
             return value
         elif type(value) in (int, float):
-            return Decimal(str(value))
+            return Decimal(six.text_type(value))
 
         if SHOULD_NOT_USE_LOCALE:
             try:
@@ -239,9 +257,9 @@ class DecimalField(Field):
         else:
             locale_vars = locale.localeconv()
             decimal_separator = locale_vars['decimal_point']
-            interesting_vars = ['decimal_point', 'mon_decimal_point',
+            interesting_vars = ('decimal_point', 'mon_decimal_point',
                                 'mon_thousands_sep', 'negative_sign',
-                                'positive_sign', 'thousands_sep']
+                                'positive_sign', 'thousands_sep')
             chars = (locale_vars[x].replace('.', r'\.').replace('-', r'\-')
                      for x in interesting_vars)
             interesting_chars = ''.join(set(chars))
@@ -277,7 +295,7 @@ class PercentField(DecimalField):
         elif value == Decimal('0'):
             return '0.00%'
 
-        value = Decimal(str(value * 100)[:-2])
+        value = Decimal(six.text_type(value * 100)[:-2])
         value = super(PercentField, cls).serialize(value, *args, **kwargs)
         return '{}%'.format(value)
 
@@ -310,7 +328,7 @@ class DateField(Field):
         if value is None:
             return ''
 
-        return types.UnicodeType(value.strftime(cls.OUTPUT_FORMAT))
+        return six.text_type(value.strftime(cls.OUTPUT_FORMAT))
 
     @classmethod
     def deserialize(cls, value, *args, **kwargs):
@@ -319,8 +337,6 @@ class DateField(Field):
             return value
 
         value = as_string(value)
-        if isinstance(value, unicode):
-            value = value.encode('utf-8')
 
         dt_object = datetime.datetime.strptime(value, cls.INPUT_FORMAT)
         return datetime.date(dt_object.year, dt_object.month, dt_object.day)
@@ -341,7 +357,7 @@ class DatetimeField(Field):
         if value is None:
             return ''
 
-        return types.UnicodeType(value.isoformat())
+        return six.text_type(value.isoformat())
 
     @classmethod
     def deserialize(cls, value, *args, **kwargs):
@@ -364,16 +380,14 @@ class TextField(Field):
     Is not locale-aware (does not need to be)
     """
 
-    TYPE = (types.UnicodeType, )
+    TYPE = (six.text_type, )
 
     @classmethod
     def deserialize(cls, value, *args, **kwargs):
         if value is None or isinstance(value, cls.TYPE):
             return value
-        elif 'encoding' in kwargs:
-            return as_string(value).decode(kwargs['encoding'])
         else:
-            return cls.TYPE[0](value)
+            return as_string(value)
 
 
 class EmailField(TextField):
@@ -390,7 +404,7 @@ class EmailField(TextField):
         if value is None:
             return ''
 
-        return types.UnicodeType(value)
+        return six.text_type(value)
 
     @classmethod
     def deserialize(cls, value, *args, **kwargs):
@@ -411,7 +425,7 @@ class JSONField(Field):
     Is not locale-aware (does not need to be)
     """
 
-    TYPE = (types.ListType, types.DictType)
+    TYPE = (list, dict)
 
     @classmethod
     def serialize(cls, value, *args, **kwargs):
@@ -419,44 +433,43 @@ class JSONField(Field):
 
     @classmethod
     def deserialize(cls, value, *args, **kwargs):
-        if isinstance(value, types.UnicodeType):
-            value = value.encode('utf-8')
-
         if value is None or isinstance(value, cls.TYPE):
             return value
         else:
             return json.loads(value)
 
 
-AVAILABLE_FIELD_TYPES = [locals()[element] for element in __all__
-                         if 'Field' in element and element != 'Field']
+local_vars = locals()
+TYPES = [(key, local_vars.get(key)) for key in __all__ if key != 'Field']
+AVAILABLE_FIELD_TYPES = [item[1] for item in TYPES]
 
 
 def as_string(value):
-    if isinstance(value, types.StringTypes):
+    if isinstance(value, six.binary_type):
+        raise ValueError('Binary is not supported')
+    elif isinstance(value, six.text_type):
         return value
     else:
-        return types.StringType(value)
+        return six.text_type(value)
 
 
 def is_null(value):
     if value is None:
         return True
-
-    value_str = as_string(value).strip().lower()
-    return not value_str or value_str in NULL
+    elif type(value) is six.binary_type:
+        value = value.strip().lower()
+        return not value or value in NULL_BYTES
+    else:
+        value_str = as_string(value).strip().lower()
+        return not value_str or value_str in NULL
 
 
 def unique_values(values):
-    value_types = set(type(value) for value in values)
-    if dict not in value_types:
-        return set([value for value in set(values) if not is_null(value)])
-    else:
-        result = []
-        for value in values:
-            if not is_null(value) and value not in result:
-                result.append(value)
-        return result
+    result = []
+    for value in values:
+        if not is_null(value) and value not in result:
+            result.append(value)
+    return result
 
 
 def detect_types(field_names, field_values, field_types=AVAILABLE_FIELD_TYPES,
@@ -467,14 +480,15 @@ def detect_types(field_names, field_values, field_types=AVAILABLE_FIELD_TYPES,
     # TODO: may receive 'type hints'
     # TODO: should support receiving unicode objects directly
     # TODO: should expect data in unicode or will be able to use binary data?
+
     field_values = list(field_values)
     if not field_values:
         return collections.OrderedDict([(field_name, BinaryField)
                                         for field_name in field_names])
 
     number_of_fields = len(field_names)
-    columns = zip(*[row for row in field_values
-                    if len(row) == number_of_fields])
+    columns = list(zip(*[row for row in field_values
+                         if len(row) == number_of_fields]))
 
     if len(columns) != number_of_fields:
         raise ValueError('Number of fields differ')
@@ -483,13 +497,16 @@ def detect_types(field_names, field_values, field_types=AVAILABLE_FIELD_TYPES,
                                               for field_name in field_names])
     for index, field_name in enumerate(field_names):
         data = unique_values(columns[index])
+        native_types = set(type(value) for value in data)
 
         if not data:
-            # all rows with an empty field -> BinaryField (can't identify)
+            # all values with an empty field (can't identify) -> BinaryField
+            identified_type = BinaryField
+        elif native_types == set([six.binary_type]):
             identified_type = BinaryField
         else:
             # ok, let's try to identify the type of this column by
-            # converting every non-null value in the sample
+            # trying to convert every non-null value in the sample
             possible_types = list(field_types)
             for value in data:
                 cant_be = set()
@@ -508,12 +525,9 @@ def detect_types(field_names, field_values, field_types=AVAILABLE_FIELD_TYPES,
     return detected_types
 
 
-TYPES = [(key, locals().get(key)) for key in __all__]
-
-
 def identify_type(value):
     value_type = type(value)
-    if value_type not in (str, unicode):
+    if value_type not in (six.text_type, six.binary_type):
         possible_types = [type_class for type_name, type_class in TYPES
                           if value_type in type_class.TYPE]
         if not possible_types:
