@@ -1,6 +1,6 @@
 # coding: utf-8
 
-# Copyright 2014-2015 Álvaro Justen <https://github.com/turicas/rows/>
+# Copyright 2014-2016 Álvaro Justen <https://github.com/turicas/rows/>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 from __future__ import unicode_literals
 
 import datetime
+
+from decimal import Decimal
 from io import BytesIO
 
 import xlrd
@@ -29,13 +31,15 @@ from rows.plugins.utils import (create_table, get_filename_and_fobj,
                                 prepare_to_export)
 
 
-CELL_TYPES = {xlrd.XL_CELL_BLANK: fields.BinaryField,
-              xlrd.XL_CELL_DATE: fields.DatetimeField,
-              xlrd.XL_CELL_ERROR: None,
-              xlrd.XL_CELL_TEXT: fields.TextField,
-              xlrd.XL_CELL_BOOLEAN: fields.BoolField,
-              xlrd.XL_CELL_EMPTY: None,
-              xlrd.XL_CELL_NUMBER: fields.FloatField,}
+CELL_TYPES = {
+        xlrd.XL_CELL_BLANK: fields.TextField,
+        xlrd.XL_CELL_DATE: fields.DatetimeField,
+        xlrd.XL_CELL_ERROR: None,
+        xlrd.XL_CELL_TEXT: fields.TextField,
+        xlrd.XL_CELL_BOOLEAN: fields.BoolField,
+        xlrd.XL_CELL_EMPTY: None,
+        xlrd.XL_CELL_NUMBER: fields.FloatField,
+}
 
 
 # TODO: add more formatting styles for other types such as currency
@@ -81,13 +85,19 @@ def cell_value(sheet, row, col):
     cell = sheet.cell(row, col)
     field_type = CELL_TYPES[cell.ctype]
 
-    if field_type is None:
-        return None
-
     # TODO: this approach will not work if using locale
     value = cell.value
 
-    if field_type is fields.DatetimeField:
+    if field_type is None:
+        return None
+
+    elif field_type is fields.TextField:
+        if cell.ctype != xlrd.XL_CELL_BLANK:
+            return value
+        else:
+            return ''
+
+    elif field_type is fields.DatetimeField:
         time_tuple = xlrd.xldate_as_tuple(value, sheet.book.datemode)
         value = field_type.serialize(datetime.datetime(*time_tuple))
         return value.split('T00:00:00')[0]
@@ -97,6 +107,7 @@ def cell_value(sheet, row, col):
             return False
         elif value == 1:
             return True
+
     else:
         book = sheet.book
         xf = book.xf_list[cell.xf_index]
@@ -106,7 +117,15 @@ def cell_value(sheet, row, col):
             # TODO: we may optimize this approach: we're converting to string
             # and the library is detecting the type when we could just say to
             # the library this value is PercentField
-            return '{}%'.format(value * 100) if value else None
+
+            if value is not None:
+                try:
+                    decimal_places = len(fmt.format_str[:-1].split('.')[-1])
+                except IndexError:
+                    decimal_places = 2
+                return '{}%'.format(str(round(value * 100, decimal_places)))
+            else:
+                return None
 
         elif type(value) == float and int(value) == value:
             return int(value)
@@ -118,7 +137,7 @@ def cell_value(sheet, row, col):
 def import_from_xls(filename_or_fobj, sheet_name=None, sheet_index=0,
                     start_row=0, start_column=0, *args, **kwargs):
 
-    filename, _ = get_filename_and_fobj(filename_or_fobj)
+    filename, _ = get_filename_and_fobj(filename_or_fobj, mode='rb')
     book = xlrd.open_workbook(filename, formatting_info=True)
     if sheet_name is not None:
         sheet = book.sheet_by_name(sheet_name)
@@ -131,7 +150,9 @@ def import_from_xls(filename_or_fobj, sheet_name=None, sheet_index=0,
                    for column_index in range(start_column, sheet.ncols)]
                   for row_index in range(start_row, sheet.nrows)]
 
-    meta = {'imported_from': 'xls', 'filename': filename,}
+    meta = {'imported_from': 'xls',
+            'filename': filename,
+            'sheet_name': sheet.name, }
     return create_table(table_rows, meta=meta, *args, **kwargs)
 
 
@@ -143,11 +164,12 @@ def export_to_xls(table, filename_or_fobj=None, sheet_name='Sheet1', *args,
 
     prepared_table = prepare_to_export(table, *args, **kwargs)
 
-    field_names = prepared_table.next()
+    field_names = next(prepared_table)
     for column_index, field_name in enumerate(field_names):
         sheet.write(0, column_index, field_name)
 
-    _convert_row = _python_to_xls(map(table.fields.get, field_names))
+    _convert_row = _python_to_xls([table.fields.get(field)
+                                   for field in field_names])
     for row_index, row in enumerate(prepared_table, start=1):
         for column_index, (value, data) in enumerate(_convert_row(row)):
             sheet.write(row_index, column_index, value, **data)
